@@ -6,7 +6,7 @@
 
     Implements proper, automated rendering for Eve responses.
 
-    :copyright: (c) 2013 by Nicola Iarocci.
+    :copyright: (c) 2014 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -16,14 +16,14 @@ import simplejson as json
 from werkzeug import utils
 from functools import wraps
 from eve.methods.common import get_rate_limit
-from eve.utils import date_to_str, config, request_method
-from flask import make_response, request, Response, current_app as app
+from eve.utils import date_to_str, config, request_method, debug_error_message
+from flask import make_response, request, Response, current_app as app, abort
 
 # mapping between supported mime types and render functions.
-_MIME_TYPES = [{'mime': ('application/json',), 'renderer': 'render_json'},
-               {'mime': ('application/xml', 'text/xml', 'application/x-xml',),
-                'renderer': 'render_xml'}]
-_DEFAULT_MIME = 'application/json'
+_MIME_TYPES = [
+    {'mime': ('application/json',), 'renderer': 'render_json', 'tag': 'JSON'},
+    {'mime': ('application/xml', 'text/xml', 'application/x-xml',),
+     'renderer': 'render_xml', 'tag': 'XML'}]
 
 
 def raise_event(f):
@@ -100,6 +100,9 @@ def _prepare_response(resource, dct, last_modified=None, etag=None,
     :param etag: ETag header value.
     :param status: response status.
 
+    .. versionchanged:: 0.3
+       Support for X_MAX_AGE.
+
     .. versionchanged:: 0.1.0
        Support for optional HATEOAS.
 
@@ -168,7 +171,7 @@ def _prepare_response(resource, dct, last_modified=None, etag=None,
         resp.headers.add('Access-Control-Allow-Origin', ', '.join(domains))
         resp.headers.add('Access-Control-Allow-Headers', ', '.join(headers))
         resp.headers.add('Access-Control-Allow-Methods', methods)
-        resp.headers.add('Access-Control-Allow-Max-Age', 21600)
+        resp.headers.add('Access-Control-Allow-Max-Age', config.X_MAX_AGE)
 
     # Rate-Limiting
     limit = get_rate_limit()
@@ -184,15 +187,26 @@ def _best_mime():
     """ Returns the best match between the requested mime type and the
     ones supported by Eve. Along with the mime, also the corresponding
     render function is returns.
+
+    .. versionchanged:: 0.3
+       Support for optional renderers via XML and JSON configuration keywords.
     """
     supported = []
     renders = {}
     for mime in _MIME_TYPES:
-        for mime_type in mime['mime']:
-            supported.append(mime_type)
-            renders[mime_type] = mime['renderer']
+        # only mime types that have not been disabled via configuration
+        if app.config.get(mime['tag'], True):
+            for mime_type in mime['mime']:
+                supported.append(mime_type)
+                renders[mime_type] = mime['renderer']
+
+    if len(supported) == 0:
+        abort(500, description=debug_error_message(
+            'Configuration error: no supported mime types')
+        )
+
     best_match = request.accept_mimetypes.best_match(supported) or \
-        _DEFAULT_MIME
+        supported[0]
     return best_match, renders[best_match]
 
 
@@ -251,7 +265,7 @@ def xml_root_open(data):
 
     .. versionadded:: 0.0.3
     """
-    links = data.get('_links')
+    links = data.get(config.LINKS)
     href = title = ''
     if links and 'self' in links:
         self_ = links.pop('self')
@@ -274,7 +288,7 @@ def xml_add_links(data):
     """
     xml = ''
     chunk = '<link rel="%s" href="%s" title="%s" />'
-    links = data.pop('_links', {})
+    links = data.pop(config.LINKS, {})
     for rel, link in links.items():
         if isinstance(link, list):
             xml += ''.join([chunk % (rel, utils.escape(d['href']), d['title'])
@@ -295,7 +309,7 @@ def xml_add_items(data):
     .. versionadded:: 0.0.3
     """
     try:
-        xml = ''.join([xml_item(item) for item in data['_items']])
+        xml = ''.join([xml_item(item) for item in data[config.ITEMS]])
     except:
         xml = xml_dict(data)
     return xml
