@@ -184,7 +184,7 @@ class TestUtils(TestBase):
     def test_debug_error_message(self):
         with self.app.test_request_context():
             self.app.config['DEBUG'] = False
-            self.assertEquals(debug_error_message('An error message'), None)
+            self.assertEqual(debug_error_message('An error message'), None)
             self.app.config['DEBUG'] = True
-            self.assertEquals(debug_error_message('An error message'),
-                              'An error message')
+            self.assertEqual(debug_error_message('An error message'),
+                             'An error message')
