@@ -16,14 +16,16 @@ class TAuth(TokenAuth):
 
 class Auth(BasicAuth):
     def check_auth(self, username, password, allowed_roles, resource, method):
-        self.request_auth_value = "nicola"
+        #print "username", username
+        self.request_auth_value = username
         return username == 'token'
         #return username == 'admin' and password == 'secret'
 
 
 class Validator(Validator):
     def _validate_msg_limit(self, msg_limit, field, value):
-        print msg_limit, field, value
+        pass
+        #print msg_limit, field, value
 
 def getting_contatti(documents):
     print ("contatti")
@@ -46,15 +48,19 @@ def resource_gotit(request, response):
     print (request.data)
     print (response.data)
 
-def resource_post(request, response):
-    print ('post_contacts')
-    print (request.authorization)
-    print (request.get_data())
-    print (response.get_data())
+def resource_post(resource, request, response):
+    print response.get_data()
+    pass
+    #print resource
+    ##print (request.authorization)
+    #print request.get_data()
+    #print response.get_data()
 
 def posting(documents):
     #print ("posting to ", resource)
-    documents[0]["token"] = "mytoken"
+    #documents[0]["token"] = "mytoken"
+    #from flask import request
+    #print "usr", request.authorization.username
     print documents
 
 def posting_r(documents):
@@ -70,14 +76,15 @@ app = Eve(auth=Auth, validator=Validator, redis=redis)
 #app.on_get_contacts += resource_gotit
 #app.on_get += gotit
 #app.on_pre_GET += getting
-#app.on_post_POST_contacts += resource_post
 #app.on_posting += posting
 #app.on_posting_contacts += posting_r
 #app.on_fetch_resource += getting
-#app.on_insert_contacts += posting
+app.on_insert_contacts += posting
 #app.on_getting_contacts += getting_contatti
+#app.on_post_POST += resource_post
 
 if __name__ == '__main__':
     #app.events.on_getting += pre
     #app.on_GET += gotit
+    #app.on_post_POST += resource_post
     app.run(debug=True)
